#include <sys/signal.h>
#include <ucontext.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>

#include "vx32.h"
#include "vx32impl.h"

static void sighandler(int signo, siginfo_t *si, void *ucontext)
{
	ucontext_t *uctx;
	
	uctx = ucontext;
	if(!vx32_sighandler(signo, si, ucontext)){
fprintf(stderr, "DEREGISTER!\n");
		// Not handled - switch back to the default handler
		// and let the signal happen again.
		signal(signo, SIG_DFL);
	}
}

// vx32_sighandler saves execution context, finds emu,
// and calls vxemu_sighandler.
void vxemu_sighandler(vxemu *emu, uint32_t trapeip)
{
if(0) fprintf(stderr, "vxemu_sighandler %x %x\n", trapeip, emu->cpu.trap);

	if(!(emu->cpu.trap & VXTRAP_INTERNAL)) {
		// Binary search for the translated chunk in which the trap occurred.
		// NB: frags appear in the opposite order as the code itself.
		uint32_t *codetab = emu->codetab;
		unsigned lofrag = 0;
		unsigned hifrag = (uint32_t*)emu->codetop - codetab - 1;
		while (hifrag > lofrag) {
			unsigned midfrag = (lofrag + hifrag) / 2;
			uint32_t mideip = codetab[midfrag];
			if (trapeip >= mideip)
				hifrag = midfrag;
			else
				lofrag = midfrag + 1;
		}
		struct vxfrag *frag = (vxfrag*)(intptr_t)codetab[lofrag];
	
		// Now binary search through this fragment's instruction table
		// for the actual faulting translated instruction,
		// and compute the corresponding EIP in the original vx32 code.
		unsigned ninsn = frag->ninsn;
		unsigned insnofs = trapeip - (uint32_t)(intptr_t)FRAGCODE(frag);
		unsigned loinsn = 0;
		unsigned hiinsn = ninsn - 1;
		while (hiinsn > loinsn) {
			unsigned midinsn = (loinsn + hiinsn + 1) / 2;
			unsigned midofs = frag->insn[midinsn].dstofs;
			if (insnofs >= midofs)
				loinsn = midinsn;
			else
				hiinsn = midinsn - 1;
		}
		struct vxinsn *insn = &frag->insn[loinsn];
	
		// Find the corresponding EIP in the original code
		emu->cpu.eip = frag->eip + insn->srcofs;
	
		// XXX fixup registers as necessary
	}else
		emu->cpu.trap &= ~VXTRAP_INTERNAL;

	siglongjmp(*emu->trapenv, 1);
}

int vx32_siginit(void)
{
	stack_t ss;
	void *stk;
	struct sigaction sa;
	
	// See if there's already an alternate signal stck.
	if (sigaltstack(NULL, &ss) < 0)
		return -1;
	assert(!(ss.ss_flags & SS_ONSTACK));
	if (ss.ss_flags & SS_DISABLE) {
		// Allocate an alternate signal stack.
		stk = malloc(SIGSTKSZ);
		if (stk == NULL)
			return -1;
		ss.ss_flags = 0;
		ss.ss_sp = stk;
		ss.ss_size = SIGSTKSZ;
		if (sigaltstack(&ss, NULL) < 0) {
			free(stk);
			return -1;
		}
	}
	
	// Register our signal handler.
	memset(&sa, 0, sizeof sa);
	sa.sa_handler = (void*)sighandler;
	sa.sa_flags = SA_ONSTACK | SA_SIGINFO;
	if (sigemptyset(&sa.sa_mask) < 0)
		return -1;
	if (sigaction(SIGSEGV, &sa, NULL) < 0)
		return -1;
	if (sigaction(SIGSEGV, &sa, NULL) < 0)
		return -1;
	if (sigaction(SIGSEGV, &sa, NULL) < 0)
		return -1;
	return 0;
}

